/*
 * gps.h
 *
 * Created 14 Nov 2013
 * Author: Joel Heck
 * 
 * Adapted from code "gps.c" created by Brian on 25 Mar 2012 for a 4760
 * project
 *
 * Defines data structures and declares functions used for GPS data processing
 */

#ifndef GPS_H_
#define GPS_H_

#include "main.h"

// Defines the maximum number of data characters in a GPS sentence that can be
// stored in the receive buffer
#define MAX_GPS_STRING_SIZE 200

// "Placeholders" to determine which part of GPS sentence is being parsed
// Listed in order as given by GPS sentence structure
enum GPS_DATA
{
	GGA_TITLE,
	GGA_UTC,
	GGA_LATITUDE,
	GGA_NS,
	GGA_LONGITUDE,
	GGA_EW,
	GGA_FIX,
	GGA_SATELLITES,
	GGA_HDOP,
	GGA_ALTITUDE,
	GGA_ALTITUDE_UNITS,
	GGA_GEOID_HEIGHT,
	GGA_GEOID_H_UNITS,
	GGA_BLANK,
	GGA_CHECKSUM,

	RMC_TITLE,
	RMC_UTC,
	RMC_STATUS,
	RMC_LATITUDE,
	RMC_NS,
	RMC_LONGITUDE,
	RMC_EW,
	RMC_SPEED,
	RMC_HEADING,
	RMC_DATE,
	RMC_MAGNETIC_VAR,
	RMC_VAR_DIRECTION,
	RMC_CHECKSUM,

	DONE
};

// Structure to hold received GPS data. Only used in "gps.c"
typedef struct gps_data {
	enum GPS_DATA gps_status;	// Indicates part of data being parsed
	uint8_t data_collected;		// Acts as boolean to indicate data received
	volatile Position* raw;		// Stores parsed GPS data
	uint8_t rx[MAX_GPS_STRING_SIZE];	// Stores raw GPS data characters
	uint8_t rx_pos;		// Index of rx[]
	enum GPS_DATA rx_status;	// Indicates part of data being received
  	uint8_t fix;		// Indicates quality of fix to satellites
} gps_data;	

// Main GPS data retrieval functions
void receive_gps_uart(char);
void init_gps(volatile Position*);
int8_t get_gps(void);

// GPS time conversion functions
uint32_t gpstime_to_sec(const uint8_t*);
//uint32_t gpstime_to_local(const uint8_t*);

// GPS location conversion functions
double gps_pos(const uint8_t*, uint8_t);
uint8_t gps_deg(const uint8_t*, uint8_t);
uint8_t gps_min(const uint8_t*, uint8_t);
uint8_t gps_sec(const uint8_t*, uint8_t);

#endif /* GPS_H_ */
